
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "MathStatRand.h"
#include "HapType.h"
#include "ranlib.h"

extern int gCtr;




/******************************************************************************************/
/*                                                                                        */
/*		              LOW-LEVEL MEMORY ALLOCATION/DEALLOCATION                            */
/*                                                                                        */  
/******************************************************************************************/

/* allocates space to a node struct, sets all its branch-pointers to NULL and  */
/*  its values to 0.0; and then returns a pointer to it  */
node_struct *AllocNode(int inID)
{
	int i,j;
	node_struct *temp;

	/* do memory allocations  */
	temp = (node_struct *)malloc(sizeof(node_struct));
	gMem.PtNS_Array[gMem.NS_ctr++] = temp;
	
	temp->Br = (branch_struct **)calloc(3 , sizeof(branch_struct *));
	temp->Pos = (double *)calloc(3 , sizeof(double));
	temp->P = (double **)calloc(3 , sizeof(double *));
		
	/* Initialize the values and allocate to the P field  */
	temp->type = Unset;
	temp->x = 0.0;
	temp->time = 0.0;
	temp->ID = inID;
	for (i=0;i<3;i++)  {
		temp->Br[i] = NULL;
		temp->Pos[i] = 0.0; 
		temp->P[i] = (double *)calloc(3,sizeof(double));
		for(j=0;j<3;j++)  {
			temp->P[i][j] = 0.0;
		}
	}
	temp->NumThrough = 0;
	
	return(temp);
}


/*  allocates memory to a branch and sets its pointers to NULL and its values to 0.0's or -1's, as appropriate. */
branch_struct *AllocBranch(parm_struct *Pars)
{
	int i;
	branch_struct *temp;
	
	temp = (branch_struct *)malloc(sizeof(branch_struct));
	gMem.PtBS_Array[gMem.BS_ctr++] = temp;
	
	temp->top = NULL;
	temp->bottom = NULL;
	
	temp->lo = 0.0;
	temp->hi = 0.0;
	
	temp->BS = UnsetB;
	temp->TS = UnsetB;
	
	temp->NumThrough = (int *)calloc(3,sizeof(int));
	for(i=0;i<3;i++) 
		temp->NumThrough[i] = 0;
		
	/*  record along with the branch, the number of leaves that are being kept track of. */
	/*  This makes de-allocating the memory easier */
	temp->NumLeaves = Pars->NumLeaves;
		
	/*  now, allocate space for the array of pointers to the intervals array here */
	temp->AncBitsBySeg = (IntervalsStruct **)calloc(Pars->NumLeaves , sizeof(IntervalsStruct *));
	
	/* and set those all to NULL */
	for(i=0;i<Pars->NumLeaves;i++)  
		temp->AncBitsBySeg[i] = NULL;
	
	return(temp);
}



/*  deallocates space that is pointed to by node_struct *N. */
/*  This also deallocates the Left and Right branches of Coalescent */
/*  Nodes and the Middle branches of Recom nodes. */
void DeallocNode(node_struct *N)
{
	int i;

	/*  just go through and reverse the steps in AllocNode() */
	for(i=0;i<3;i++)
		free(N->P[i]);
	free(N->P);
	free(N->Pos);
	free(N->Br);
	free(N);
}




/*  de-allocates memory to a branch */
void DeAllocBranch(branch_struct *B)
{
	int i;
	
	/*  basically just reverse AllocBranch() and the effects of AllocIntervals() */
	DestroyIntervals(B->AncBits);
	DestroyIntervals(B->OverallAncBits);
	for(i=0;i<B->NumLeaves;i++)  {
		DestroyIntervals(B->AncBitsBySeg[i]);
	}
	free(B->AncBitsBySeg);
	free(B->NumThrough);
	free(B);
	B = NULL;
}




/******************************************************************************************/
/*                                                                                        */
/*		                PRINTING AND SIMPLE OUTPUT FUNCTIONS                              */
/*                                                                                        */  
/******************************************************************************************/
/*  prints the information about a branch onto one line */
void PrintBranch(branch_struct *B)
{
	if(B==NULL) {
		printf("NULL");
	}
	else {
		if(B->top != NULL)
			printf("Top->%d",B->top->ID);
		else 
			printf("Top->NULL");
		if(B->bottom != NULL)
			printf("  Bottom->%d",B->bottom->ID);
		else 
			printf("  Bottom->NULL");
			
/* 		printf("  Lo=%.5f  Hi=%.5f, BS = %s, TS = %s  NumThrough[0,1,2] = (%d, %d, %d)",B->lo,B->hi, BranchNum2str(B->BS), */
/* 					 BranchNum2str(B->TS), B->NumThrough[0],B->NumThrough[1],B->NumThrough[2]); */

		PrintIntervals(B->OverallAncBits);
	}
}

void PrintBranchList(branch_list *B)
{
	branch_list *temp;
	
	temp = B;
	
	while(temp != NULL)  {
		printf("\n\t");
		PrintBranch(temp->Data);
		temp = temp->Next;
	}
}

/*  prints the information about a node on a line, and then  */
/*  prints the info about its branches on three new lines beneath if */
void PrintNode(node_struct *N)
{
	int i;
	
	if(N==NULL)  {
		printf("NODE is NULL");
	}
	else {
		printf("Node: ID=%d  Type=%s  Time=%.5f  x=%.5f",N->ID,NodeType2str(N->type),N->time,N->x);
		printf("     PosLeft = %.4f PosMid = %.4f PosRight = %.4f", N->P[Left][0], N->P[Middle][0], N->P[Right][0]);
		printf("  NumThrough = %d", N->NumThrough);
		
		for(i=0;i<3;i++)  {
			printf("\n\t%s Branch:  ",BranchNum2str(i));
			PrintBranch(N->Br[i]);
		}
	
	}
}

/*  converts a NodeType enum to a string */
const char *NodeType2str(enum NodeType node)
{

	switch (node) {
	case Unset: return "Unset";
	case Leaf: return "Leaf";
	case Recomb: return "Recomb";
	case Coalesce: return "Coalesce";
	default: return "!!Error-Unknown Locus Type!!";
	}
}

/*  converts a BranchNums enum to a string */
const char *BranchNum2str(enum BranchNums nums)
{

	switch (nums) {
	case Left: return "Left";
	case Right: return "Right";
	case Middle: return "Middle";
	case UnsetB: return "Unset";
	default: return "!!Error-Unknown BranchNum!!";
	}
}






/******************************************************************************************/
/*                                                                                        */
/*		                     GENEALOGY SIMULATION FUNCTIONS                               */
/*                                -- Nuts and Bolts --                                    */  
/******************************************************************************************/


/* allocates space to a new element in the list and inserts it at the */
/*  front of the list (nearest B), and then it returns a pointer to that  */
/*  front end.  So this should be called via sometime like */
/*  B = AppendBranchToList(B,S); */
/*  If you pass it a NULL pointer then this still works as the initial */
/*  placement.  So, always set your pointers to NULL before you do anything with them  */
branch_list *AppendBranchToList(branch_list *B, branch_struct *StickItInThere) 
{	
	branch_list *temp;
	
	temp = (branch_list *)malloc(sizeof(branch_list));
	temp->Data = StickItInThere;
	temp->Next = B;
	
	
	return(temp);
}


IntervalsStruct *UnionOverBranches(branch_list *B)
{
	branch_list *tempB;
	IntervalsStruct *temp, *dtemp;
	
	/*  define dtemp to be a copy of B->Data->OverallAncBits. */
	temp = CopyIntervals(B->Data->OverallAncBits);
	
	/*  then cycle through the list, taking unions as we go. */
	tempB = B;
	while(tempB->Next != NULL)  {
		dtemp = temp;
		tempB = tempB->Next;
		temp = IntervalsUnion(dtemp, tempB->Data->OverallAncBits);
		DestroyIntervals(dtemp);
	}
	
	
	return(temp);
}


/*  given a recombination point at x, this returns the intersection of the current */
/*  AncBits with [Pars->Lo,x] or [x,Pars->Hi] depending on whether this is Left or Right */
IntervalsStruct *RecombIntersect(IntervalsStruct *InAnc, double x, enum BranchNums RightLeft, parm_struct *Pars)
{
	IntervalsStruct *temp, *sendback;
	
	temp = AllocIntervals(2);
	
	if(RightLeft == Left)  {
		temp->B[0].v = Pars->Lo;
		temp->B[1].v = x;
	}
	else {
		temp->B[0].v = x;
		temp->B[1].v = Pars->Hi;
	}
	
	sendback = IntervalsIntersection(temp,InAnc);
	
	DestroyIntervals(temp);
	
	return(sendback);
}





/* allocates to an interval struct for a leaf which will either have nothing */
/*  in it, or it will go from Pars->Lo to Pars->Hi, depending on if it is  */
/*  On or Off.  */
IntervalsStruct *CreateLeafInterval(int OnOff, parm_struct *Pars)
{
	IntervalsStruct *temp;
	
	
	
	if(OnOff)  {  /* if it is "On"  (i.e. non-zero)  */
		temp = AllocIntervals(2);
		temp->lo = Pars->Lo;
		temp->hi = Pars->Hi;
		temp->B[0].v = Pars->Lo;
		temp->B[1].v = Pars->Hi;
	}
	else {
		temp = AllocIntervals(0);
	}

	return(temp);

}




/*  allocates space and initializes values for a leaf, with chromosomes that */
/*  are defined from real numbers Lo to Hi */
node_struct *CreateLeaf(int Idx, parm_struct *Pars)
{
	int i, ID = -Idx - 1;
	node_struct *temp;
	double Lo = Pars->Lo;
	double Hi = Pars->Hi;
	
	temp = AllocNode(ID);
	temp->type = Leaf;
	
	temp->Br[Middle] = AllocBranch(Pars);
	temp->Br[Middle]->bottom = temp;
	temp->Br[Middle]->BS = Middle;
	
	temp->Br[Middle]->lo = Lo;
	temp->Br[Middle]->hi = Hi;
	
	temp->Br[Middle]->wt = Hi - Lo;
	
	/*  allocate and initialize the intervals on the branch coming out the top, too */
	temp->Br[Middle]->AncBits = CreateLeafInterval(1,Pars);
	
	temp->Br[Middle]->OverallAncBits = CreateLeafInterval(0,Pars);
	for(i=0;i<Pars->NumLeaves;i++)  {
		/*  make an array.  Only the interval for which i==Idx is "On" */
		temp->Br[Middle]->AncBitsBySeg[i] = CreateLeafInterval( (i==Idx) ,Pars);
	}
	
	

	return(temp);
}








/* given a time, and the branch list and the number of branches, this function  */
/*  chooses a random pair of the branches and then coalesces them, updates the */
/*  branch list and then returns a pointer to the new node.
	This function also decrements NumBranches.  If it is a normal coalescence 
	then the number of branches gets decremented by one.  If, however, the coalescence
	results in a middle branch that has no more ancestral material actually waiting for 
	a common ancestor (i.e. it has weight zero) then the NumBranches gets decremented by
	two and BOTH branches coming in to the coalescence get dropped from the current branch
	list */
node_struct *CoalesceTwoRandomBranches(branch_list **Bl, double Time, int *NumBranches, int ID, parm_struct *Pars)
{
	int a,b,i;  
	branch_list *A,*B, *temp, *preB, *preA;
	node_struct *TempNS;
	
	/* first choose the subscripts of the branches to be taken.  Assume they are  */
	/*  ordered from 0 to NumBranches-1 */ 
	OrderedPairRV(0,*NumBranches-1,&a,&b);
	
	/* then set the A and B pointers to the appropriate branch_list elements
		also keep track of the elements before A and B so that we can drop
		them from the list if necessary */
	temp = *Bl;
	for(i=0;i<=b;i++)  {
		if(a>0 && i == a-1)
			preA = temp;
		if(i==a) 
			A = temp;
		if(i==b-1)
			preB = temp;
		if(i==b)
			B = temp;
		temp = temp->Next;
	}
	
	/* then coalesce those dudes */
	TempNS = CreateCoalescence(A->Data,B->Data,Time,ID,Pars);
	
	/* now we have to alter the branch list.  We drop the B element  */
	/*  from the list, simply by pointing preB to the element after B, */
	/*  and then we point the A element to the new branch coming */
	/*  out of the top middle of TempNS */
	preB->Next = B->Next;
	A->Data = TempNS->Br[Middle];
	
	/*  as the final hurrah,  WE MUST free the branch_list struct memory that  */
	/*  B points to.  Otherwise it will be leaked. */
	free(B);
	
	/*  Decrement NumBranches */
	*NumBranches -= 1;
	
	/*  and now, if the middle branch has weight zero we remove it from the branch list too */
	if( BranchIsActive(TempNS->Br[Middle], Pars->Lo, Pars->Hi) == 0) {
		if(a>0) {
			preA->Next = A->Next;
		}
		else {  /*  in this case, A is the first element of the BrList */
			*Bl = (*Bl)->Next;
		}
		free(A);
		*NumBranches -= 1;
	}
	 
	
	return(TempNS);
}



/* given a time, and the branch list and the number of branches, and */
/*  the length of the segment we are dealing with, this function */
/*  chooses a random branch WEIGHTED BY THE LENGHT OF ANCESTRAL OR */
/*  TRAPPED MATERIAL ON IT (stored in the branch field "wt") */
/*  and splits it at a recombination node, updates the */
/*  branch list and then returns a pointer to the new node */
/*  Bl is a branch_list ** because we need to change the thing that Bl points to. */
/*  Note that sometimes a recombination will lead to only only "active" */
/*  branch.  This function deals with that by deleting the dead branch from */
/*  memory, and it updates NumBranches appropriately. */
node_struct *SplitOneRandomBranch(branch_list **Bl, double Time, int *NumBranches,
			 int ID, double *Position, parm_struct *Pars)
{
	int a,i;  
	branch_list *A, *temp;
	node_struct *TempNS;
	double x, sum=0.0;
	double *Probs;
	
	
	/* first choose the subscript of the branch to be split.  Assume the current branches are  */
	/*  ordered from 0 to NumBranches-1.  We have to draw them according to their weights, so, cycle */
	/*  through them, copying their weights, into a temporary array then normalize those weights */
	/*  so they sum to one, then draw the random integer */
	Probs = (double *)calloc(*NumBranches, sizeof(double));
	temp = *Bl;
	for(i=0;i<*NumBranches;i++)  { 
		Probs[i] = temp->Data->wt;
		sum += Probs[i];
		temp = temp->Next;
	}
	/* now normalize the weights into probabilities */
	for(i=0;i<*NumBranches;i++)  { 
		Probs[i] /= sum;
	}
	/* draw the random integer: */
	a = IntFromProbsRV(Probs,0,*NumBranches);
	
	/* then set the A pointer to the appropriate branch_list element */
	temp = *Bl;
	for(i=0;i<a;i++)  {  /* after this loop runs, temp points to the base-0 a-th element of the  */
						 /*  list. */
		temp = temp->Next;
	}
	A = temp;
	
	/* choose a random location for the breakpoint to occur.  It must occur between the lo and  */
	/*  the hi field of the branch.  And it will be uniformly distributed on that interval */
	x = A->Data->lo + (double)ranf() * A->Data->wt;
	*Position = x;
	
	/* then recombine those dudes */
	TempNS = CreateRecomb(A->Data,x,Time,ID,Pars);
	
	/* now we have to check to see if one or both branches  */
	/*  are active and  alter the branch list and NumBranches accordingly. */
	
	
	
	if(TempNS->Br[Left]->wt > 0.0 &&  TempNS->Br[Right]->wt > 0.0)   { /* if both branches are active  */
		/*  we point the A element to the new branch coming */
		/*  out of the top left of TempNS, and we add append the */
		/*  branch coming out of the top right of TempNS to the BranchList */
		A->Data = TempNS->Br[Left];
		*Bl = AppendBranchToList(*Bl,TempNS->Br[Right]);
		/*  we now have one extra branch, too. */
		*NumBranches += 1;
	}
	else if(TempNS->Br[Left]->wt > 0.0)  {  /* in this case, only the left branch is active  */
		/*  make the input branch point to the new active branch.  NumBranches does not get changed! */
		A->Data = TempNS->Br[Left];
		/*  Also, we delete the Right branch */
		DeAllocBranch(TempNS->Br[Right]);
	}
	else if(TempNS->Br[Right]->wt > 0.0)  {  /* in this case, only the right branch is active  */
		/*  make the input branch point to the new active branch.  NumBranches does not get changed! */
		A->Data = TempNS->Br[Right];
		/* Also, we delete the left branch  */
		DeAllocBranch(TempNS->Br[Left]);
	}
	
	/* free the array  */
	free(Probs);
	
	
	return(TempNS);
}






















/*  Create a new node given two branches coming in to it and set it to be a coalescence */
/*  of branches from A and B at time t.  This involves pointing the  */
/*  pointers of various nodes to and from pointers in the branches, and also  */
/*  setting the "existence criteria" in the middle branch (continuing out the top) by way of the */
/*  lo and hi fields.  A is the "Left" incoming branch and B is the "Right" one */
node_struct *CreateCoalescence(branch_struct *A, branch_struct *B, double t, int ID, parm_struct *Pars) 
{
	int i;
	node_struct *temp;
	IntervalsStruct *TempIS;
	
	temp = AllocNode(ID);
	
	temp->type = Coalesce;
	
	temp->time = t;
	
	/*  set the pointers to and from A and B */
	temp->Br[Left] = A;
	temp->Br[Left]->top = temp;
	temp->Br[Left]->TS = Left;
	temp->Br[Right] = B;
	temp->Br[Right]->top = temp;
	temp->Br[Right]->TS = Right;
	
	/*  now, we must set the existence criteria for the branch coming out of the top */
	/*  of this node.   There will be a branch coming out of it, in a sense, even if */
	/*  the node itself is not a Coalescence for some value along the chromosome.  But, */
	/*  even if it is not a Coalescence, it can still be a continuation of a branch.   */
	/*  Therefore, we set the existence criteria for the middle branch to the union of the */
	/*  two incoming branches */
	
	/*  first allocate memory to the new branch and set its bottom pointer to the new node */
	temp->Br[Middle] = AllocBranch(Pars);
	temp->Br[Middle]->bottom = temp;
	temp->Br[Middle]->BS = Middle;
	
	/*  then determine the intervals on the new branch that contain ancestral bits. */
	/*  This is merely the union of the intervals on the incoming branches */
	temp->Br[Middle]->AncBits = IntervalsUnion(temp->Br[Left]->AncBits,temp->Br[Right]->AncBits);
	
	/*  Do that for the AncBitsBySeq too: */
	for(i=0;i<Pars->NumLeaves;i++)  {
		temp->Br[Middle]->AncBitsBySeg[i] = IntervalsUnion(temp->Br[Left]->AncBitsBySeg[i],
														temp->Br[Right]->AncBitsBySeg[i]);
	}
	/*  Then find the intersection of all those: */
	temp->Br[Middle]->OverallAncBits = IntersectOfArrayOfIntervals(temp->Br[Middle]->AncBitsBySeg,Pars->NumLeaves);

/*printf("\n\n\nAncBits ");	
PrintIntervals(temp->Br[Middle]->AncBits);
printf("\nOAncBit ");
PrintIntervals(temp->Br[Middle]->OverallAncBits);
printf("\nResult  ");
*/

	
	/*  BIG EXTRA STEP HERE!!!   */
	/*  DELETE THE PARTS OF AncBits THAT HAVE ALREADY FOUND A COMMON ANCESTOR: */
	TempIS = IntervalsSubtract(temp->Br[Middle]->AncBits, temp->Br[Middle]->OverallAncBits);
	/*  Delete the current AncBits */
	DestroyIntervals(temp->Br[Middle]->AncBits);
	/*  then point it back to TempIS */
	temp->Br[Middle]->AncBits = TempIS;
/* PrintIntervals(temp->Br[Middle]->AncBits); */
/* printf("\nDsGone  "); */
	RemoveEmptyIntervals(&(temp->Br[Middle]->AncBits));
/* PrintIntervals(temp->Br[Middle]->AncBits); */
/* fflush(stdout); */
/* scanf("%d",&boing); */
	/*  then set the lo and the hi for the branches.  This is redundant.  Eventually */
	/*  I will rewrite the progs to just use the lo and the hi in the AncBits field */
	temp->Br[Middle]->lo = temp->Br[Middle]->AncBits->lo;
	temp->Br[Middle]->hi = temp->Br[Middle]->AncBits->hi;

	
	/*  then set the weight: */
	temp->Br[Middle]->wt = temp->Br[Middle]->hi - temp->Br[Middle]->lo;
	
	return(temp);
}	


/*  Create a new node given branch A coming in to it and set it to be a recombination node. */
/*  The recombination point is x, and the time the node occurs is t */
node_struct *CreateRecomb(branch_struct *A, double x, double t, int ID, parm_struct *Pars) 
{
	int i;
	node_struct *temp;
	
	temp = AllocNode(ID);
	
	temp->type = Recomb;
	
	temp->time = t;
	
	temp->x = x;
	
	/*  set the pointers to and from A and B */
	temp->Br[Middle] = A;
	temp->Br[Middle]->top = temp;
	temp->Br[Middle]->TS = Middle;
	
	/*  now, we must set the existence criteria for the two branches coming out of the top */
	/*  of this node.   Whether or not either of the two branches is active depends on whether */
	/*  the incoming branch is active AND (intersection) the outgoing branch is left or right and the  */
	/*  position on the chromosome relative to the recombination point x. */
	
	/*  first allocate memory to the new branches and set their bottom pointers to the new node */
	temp->Br[Left] = AllocBranch(Pars);
	temp->Br[Left]->bottom = temp;
	temp->Br[Left]->BS = Left;
	temp->Br[Right] = AllocBranch(Pars);
	temp->Br[Right]->bottom = temp;
	temp->Br[Right]->BS = Right;
	
	/*  then determine the new lo and hi values.  This is basically the */
	/*  intersection of the interval [0,x] with [lo,hi]  for the left */
	/*  and [x,1] with [lo,hi] for the right branch.   */
	temp->Br[Left]->AncBits = RecombIntersect(temp->Br[Middle]->AncBits,temp->x,Left,Pars);
	temp->Br[Right]->AncBits = RecombIntersect(temp->Br[Middle]->AncBits,temp->x,Right,Pars);
	
	temp->Br[Left]->lo = temp->Br[Left]->AncBits->lo;
	temp->Br[Right]->lo = temp->Br[Right]->AncBits->lo;
	
	temp->Br[Left]->hi = temp->Br[Left]->AncBits->hi;
	temp->Br[Right]->hi = temp->Br[Right]->AncBits->hi;
	
	
	temp->Br[Left]->wt = temp->Br[Left]->AncBits->hi - temp->Br[Left]->AncBits->lo;
	temp->Br[Right]->wt = temp->Br[Right]->AncBits->hi - temp->Br[Right]->AncBits->lo;
	
	
	/*  Here we deal with the AncBitsBySeg */
	for(i=0;i<Pars->NumLeaves;i++)  {
		temp->Br[Left]->AncBitsBySeg[i] = RecombIntersect(temp->Br[Middle]->AncBitsBySeg[i],temp->x,Left,Pars);
		temp->Br[Right]->AncBitsBySeg[i] = RecombIntersect(temp->Br[Middle]->AncBitsBySeg[i],temp->x,Right,Pars);
	}
	
	/*  Then find the intersection of all those: */
	temp->Br[Left]->OverallAncBits = IntersectOfArrayOfIntervals(temp->Br[Left]->AncBitsBySeg,Pars->NumLeaves);
	temp->Br[Right]->OverallAncBits  = IntersectOfArrayOfIntervals(temp->Br[Right]->AncBitsBySeg,Pars->NumLeaves);
	
	return(temp);
}	






/******************************************************************************************/
/*                                                                                        */
/*		                   GENEALOGY SIMULATION/DESTRUCTION                               */
/*                             -- Big Ol' Functions --                                    */  
/******************************************************************************************/

/*  this function creates a new AR_graph given a set of parameters. */
/*  This involves memory allocation and simulation. */
/*  This version uses the */
/*  observation that the birthrate should depend on the amount of ancestral */
/*  or "trapped" material available to each branch */
AR_graph_struct *NewAR_graph(parm_struct *P)
{
	int i,j;
	double birthrate, deathrate, ExpMean;  /*  birth and death rates of the process  */
										   /*  and the exponential waiting time mean to the next event */
	double ProbCoal,rando;  /*  the probability that the event was a coalescence given that one occurred */
	double expRV, TotalTime = 0.0;  /*  the exponential waiting time random variable */
									/*  and the total time so far. */
	int NumBranches = 0;  /*  used while simulating the tree */
	branch_list *temp;
	double sum_wt;
	int stop = 0;
	int ItWasACoalescence;
	double CoalMean, RecombMean;
	double CoalWait, RecombWait;
	double tr, tch, tc;
	
	AR_graph_struct *A;
	
	/*  deal first with allocating to and setting up the AR_graph_struct and doing initializations */
	A = (AR_graph_struct *)malloc(sizeof(AR_graph_struct));
	
	A->Pars = P;
	A->NumEvents = 0;
	A->Events = (node_struct **)calloc(EVENT_ALLOC_BLOCK , sizeof(node_struct *));
	A->Leaves = (node_struct **)calloc(A->Pars->NumLeaves , sizeof(node_struct *));
	A->Root = NULL;
	A->BrList = NULL;
	A->NumAllBreaks = 0;
	A->AllBreaks = (double *)calloc(BREAK_ALLOC_BLOCK , sizeof(double));
	A->NumVisiBreaks = 0;
/* A->NumBreaksInAncBits = 0; */
	A->AllAncBits = (IntervalsStruct **)calloc(EVENT_ALLOC_BLOCK , sizeof(IntervalsStruct *));
	
	/*  now we basically have to simulate the thing.  */
	/*  First we set up the leaves and get the BrList pointing to them. */
	/*  Leaves get Node ID Numbers from -NumLeaves to -1 */
	for(i=0;i<A->Pars->NumLeaves;i++)  {
		A->Leaves[i] = CreateLeaf(i,A->Pars);
		A->BrList = AppendBranchToList(A->BrList, A->Leaves[i]->Br[Middle]);
		NumBranches++;
	}
	
	 


	/*   Then enter a while loop to simulate the ancestral recombination graph */
	i=0;  /*   i will count up for the ID numbers of the nodes */
	while(!stop) {
		/*  take the current branch list and take the union of the OverallAncBits over all branches */
		A->AllAncBits[i] = UnionOverBranches(A->BrList);
		
		/*  Put a little stopping condition here:  If the AllAncBits is the whole Interval, then */
		/*  we will stop after making one more node */
		if(A->AllAncBits[i]->NumBounds == 2 &&
		   A->AllAncBits[i]->B[0].v == A->Pars->Lo &&
		   A->AllAncBits[i]->B[1].v == A->Pars->Hi)  {
		  	
		  	stop = 1; 
		 }
		
		if(!stop) {
			
			/*  establish the birth and death rates  */
			deathrate =(double)(NumBranches*(NumBranches-1))/2.0;
			
			/*  the birth rate depends on the length of ancestral and trapped intervals */
			/*  on all the branches. */
			sum_wt = 0.0;  /*  Prepare to accumulate a sum of the wt's */
			temp = A->BrList;
			for(j=0;j<NumBranches;j++)  {
				sum_wt += temp->Data->wt;
				temp = temp->Next;
			}
			birthrate = sum_wt * A->Pars->rho/2.0;
			
			
			
			/*  if there is no popsize information, we do it this way... */
			if(P->pc->UsePopGrowth == 0) {
				/*  using the birth and death rates, also compute the mean waiting time */
				/*  to the next event and the probability that the next event is a Coalescent */
				ExpMean = (1.0/(birthrate + deathrate));
				ProbCoal = deathrate * ExpMean;
				/*  then draw the waiting time and a random variable to tell what type of event it was */
				rando = (double)ranf();
				expRV = genexp(ExpMean);
				/*  Then carry out the implied actions in the Ancestral Recombination graph */
				TotalTime += expRV;
				
				if(rando < ProbCoal)
					ItWasACoalescence = 1;
				else
					ItWasACoalescence = 0;
				
			}		
			/*  otherwise we set it up this way: draw two events---a coalescence in homogeneous time at time th */
			/*  and a recombination in the regular time at time t.  Then transform th into the t scale and call */
			/*  that tstar.  If tstar < th then it was a coalescence.  There is some hopping about that must be done */
			/*  to compute the total times from the waiting times in the two time scales */
			else {
				CoalMean = 1.0/deathrate;
				CoalWait = genexp(CoalMean);
				tch = Th_From_T(P->pc,TotalTime, 2.0 * P->pc->P[0].Nplus) + CoalWait;  /*  the total time to that coalescent in the "homogeneous */
													  /*  coalescent rate" time scale */
				tc = T_From_Th(P->pc, tch, 2.0 * P->pc->P[0].Nplus);    	/*  the total time to that coalescent in the "inhomogeneous */
												/*  coalescent rate" time scale. */
				
				
				if(birthrate > 0.0)  {  /*  only do this if there is some possibility of recombination. */
					RecombMean = 1.0/birthrate;
					RecombWait = genexp(RecombMean);
					
					tr = TotalTime + RecombWait;
					
					if(tr<tc)  {
						ItWasACoalescence = 0;
						TotalTime = tr;
					}
					else  {
						ItWasACoalescence = 1;
						TotalTime = tc;
					}
				}
				else {  /*  if recombination rate is zero, then it was definitely a coalescence */
					ItWasACoalescence = 1;
					TotalTime = tc;
				}
			
			}
			
			
/* if(i>=32) {printf("\nEstabblished type of event, i = %d",i); scanf("%d",&boing);}			 */
			
			
			
			
			
			
			if(ItWasACoalescence) {
				A->Events[i] = CoalesceTwoRandomBranches(&(A->BrList),TotalTime,&NumBranches,i,A->Pars);
			}
			else {  /*  in here we make a Recombination and record its position in the AllBreaks array */
				A->Events[i] = SplitOneRandomBranch(&(A->BrList),TotalTime,&NumBranches,
								i,&(A->AllBreaks[A->NumAllBreaks]), A->Pars  );
				A->AllBreaks[A->NumAllBreaks] = A->Events[i]->x;
				A->NumAllBreaks++;
/*				if( PointIsInActiveInterval(A->Events[i]->x,A->Events[i]->Br[Middle]->AncBits) )
				{
					A->BreaksInAncBits[A->NumBreaksInAncBits] = A->Events[i]->x;
					A->NumBreaksInAncBits++;
				}
*/
			}

	/* 		printf("\n"); */
	/* 		PrintNode(A->Events[i]); */
	/* 		printf("\n"); */
	/* 		PrintBranchList(A->BrList); */
		
			i++;   /*  increment the ID number for the nodes */
			
			
/* scanf("%d",&boing); */


			/*  exit with a warning if we are going to run out of room for the events */
			if(i>EVENT_ALLOC_BLOCK - 5)  {
				printf("\n\nRunning out of memory.  EVENT_ALLOC_BLOCK == %d\n\n",EVENT_ALLOC_BLOCK);
				printf("\nExiting to system...\n");
				exit(1);	
			}
			if(A->NumAllBreaks > BREAK_ALLOC_BLOCK - 5)  {
				printf("\n\nRunning out of memory.  BREAK_ALLOC_BLOCK == %d\n\n",BREAK_ALLOC_BLOCK);
				printf("\nExiting to system...\n");
				exit(1);	
			}
		}  /*  closes the if(!stop) loop. */
		
		/*  HERE WE HAVE A DIFFERENT STOPPING CONDITION THAT COMES ABOUT AS A RESULT OF KILLING */
		/*  OFF BRANCHES THAT HAVE ZERO WEIGHT.  IF NumBranches == 0, THEN WE ARE DONE SIMULATING */
		/*  THE Genealogy HUD */
		if(NumBranches==0)
			stop = 1;
		
	}  /*  closes the while loop that simulates the ancestral recombination graphs */
	
	/*  once we are done with that we set the number of events and the Root pointer */
	A->NumEvents = i;
	
	A->Root = A->Events[i-1];
	
	/*  then we sort the array of all the breaks */
	qsort(A->AllBreaks,A->NumAllBreaks,sizeof(double),CompDoubles);
	
	FindVisiBreaks(A);
	
/* qsort(A->BreaksInAncBits,A->NumBreaksInAncBits,sizeof(double),CompDoubles); */

	/* printf("\n\nNumVisiBreaks = %d  NumAllBreaks = %d\n",A->NumVisiBreaks,A->NumAllBreaks); */
	/* for(i=0;i<A->NumVisiBreaks;i++)  { */
	/* printf("\n%.5f    %.5f   ",A->VisiBreaks[i], A->AllBreaks[i]); */
	/* } */
	/* scanf("%d",&i); */
  
	return(A);
}


/*  deallocates the memory pointed to by the gMem */
void ObliterateNodesAndBranches(void)
{
	int i,j;
	
	for(i=0;i<gMem.NS_ctr;i++)  {
		DeallocNode(gMem.PtNS_Array[i]);
	}
	
	for(j=0;j<gMem.BS_ctr;j++)  {
		DeAllocBranch(gMem.PtBS_Array[j]);
	}
	
	/*  Then set the counters back to zero to be used again */
	gMem.NS_ctr = 0;
	gMem.BS_ctr = 0;
}



/*  This returns all the memory allocated to an AR-Graph to the heap */
/*  Note: we don't touch the Pars field, because that points to something */
/*  that is beyond the scope of just the current graph. */
void Destroy_AR_graph(AR_graph_struct *A)
{
	int i;
	branch_list *t, *p;
	
	
	/*  working somewhat backward relative to NewAR_graph() */
	free(A->VisiBreaks);
	free(A->MRCAofVisiBreaks);
	
	for(i=0;i<=A->NumEvents;i++)  {
		/*  blow away the AllAncBits  */
		DestroyIntervals(A->AllAncBits[i]);
	}

	
	/*  deallocate the event pointers and the branch pointers
		that got allocated during the while loop in NewAR_graph() */
	ObliterateNodesAndBranches();
	
	
	
	/*  deallocate BrList in linked list destruction fashion */
	for(p=A->BrList; p ; p=t) {
		t = p->Next;
		free(p);
	}
	
	/*  And now we deallocate all the (somewhat simpler) arrays of pointers and standard */
	/*  data types: */
	free(A->Events);
	free(A->Leaves);
	free(A->AllBreaks);
	free(A->AllAncBits);
	
	/*  at the very end, free the memory pointed to by A */
	free(A);
}



/******************************************************************************************/
/*                                                                                        */
/*		          ANALYZING GENEALOGY AFTER IT'S BEEN SIMULATED                            */
/*                                                                                        */  
/******************************************************************************************/

/*  returns the subscript of the event at which the final coalescence for a segment */
/*  on the interval [Lo,Hi] occurs.  It does this by starting at the root of the AR_graph */
/*  and then working forward in time until it finds the first coalescence in which both  */
/*  the left and right branches are active on the interval [Lo, Hi] */
/*  I THINK THAT THIS MUST BE WRONG */
int FinalCoalesce(AR_graph_struct *A, double Lo, double Hi)
{
	int i;
	
	for(i=A->NumEvents-1; i>=0; i--)  {
		if(A->Events[i]->type == Coalesce  &&
			BranchIsActive(A->Events[i]->Br[Left],Lo,Hi) && BranchIsActive(A->Events[i]->Br[Right],Lo,Hi) )  {
			
				break;
		}
	}
	
	return(i);	

}

/*  returns a one if the branch is active for any point on the interval [Lo,Hi] */
/*  and a zero otherwise */
/*  NOTE:  IF B->lo > B->hi then the branch is inactive */
int BranchIsActive(branch_struct *B, double Lo, double Hi)
{
	int temp = 0;
	
	if(Lo>Hi) {
		printf("\n\nExiting to System!  Lo > Hi in BranchIsActive()\n\n");
	}
	
	if(B->lo >= B->hi)   /*  if the lo and hi criteria for the branch are reverse ordered or equal */
						 /*  then the branch does not exist. */
		temp = 0;
	
	else if( (B->lo <= Lo && B->hi >= Lo)  || (B->lo <= Hi && B->hi >= Hi)  
		|| (Lo <= B->lo && Hi >= B->hi) )
		temp = 1;
		
	return(temp);
	
}


void SetNumThroughsToZero(AR_graph_struct *AR, int ZeroOneTwo)
{
	int i;
	
	CYC_EVENTS(i,AR)
	
		/*  set node to zero */
		AR->Events[i]->NumThrough = 0;
	
		/*  then deal with NumThrough in the branches */
		if(AR->Events[i]->type == Coalesce) {
			AR->Events[i]->Br[Left]->NumThrough[ZeroOneTwo] = 0;
			AR->Events[i]->Br[Right]->NumThrough[ZeroOneTwo] = 0;
		}
		else if(AR->Events[i]->type == Recomb)  {
			AR->Events[i]->Br[Middle]->NumThrough[ZeroOneTwo] = 0;
		}
			
	END1CYCLE

}

/*  returns a one if the genealogy represented by NumThrough[0] and NumThrough[1] */
/*  in the branches is different */
int GenealogyHasChanged(AR_graph_struct *A, int MRCA1, int MRCA2)
{
	int i;
	int temp = 0;
	
	
	if(MRCA1 != MRCA2) 
		return(1);
	else {  /*  this is currently totally inefficient---it goes through every branch beneath each */
			/*  Coalescence or each Recomb event below the common MRCA and compares the NumThroughs */
		
		for(i=MRCA1;i>=0;i--)  {
			if(A->Events[i]->type == Coalesce) {
				if( (A->Events[i]->Br[Left]->NumThrough[0] != A->Events[i]->Br[Left]->NumThrough[1])  ||
					(A->Events[i]->Br[Right]->NumThrough[0] != A->Events[i]->Br[Right]->NumThrough[1]) ) {
					
					temp = 1;
					i=-1;  /*  it will exit loop this way (in case the break doesn't get us far enough out) */
					break;  
				}
			}
			else if(A->Events[i]->type == Recomb)  {
				if(A->Events[i]->Br[Middle]->NumThrough[0] != A->Events[i]->Br[Middle]->NumThrough[1] ) {
					
					temp = 1;
					i=-1;  /*  it will exit loop this way (in case the break doesn't get us far enough out) */
					break;  
				}
			}
		}
	}
	
	return(temp);
}





/*  this traces paths back from the leaves to find the MRCA for position x. */
/*  Along the way, this counts the number of traversals through each branch */
/*  in the branch field NumThrough[OneOrZero] and in the NumThrough[2] field */
/*  OneOrZero should be 1 for odd numbered segments and 0 for even-numbered segs */
int FindMRCAbyPaths(AR_graph_struct *A, double x, int OneOrZero)
{
	int i,j;
	node_struct **NodeArray;  
	
	/*  allocate memory to NodeArray and point them at the leaves */
	NodeArray = (node_struct **)calloc(A->Pars->NumLeaves , sizeof(node_struct *));
	for(j=0;j<A->Pars->NumLeaves;j++)  {
		NodeArray[j] = A->Leaves[j];
	}
	
	/*  set all the relevant NumThroughs to zero */
	SetNumThroughsToZero(A,OneOrZero);
	
	/*  then cycle over the events, inching our way upward until we find one that */
	/*  all NumLeaves paths have gone through.  This goes to NumEvents - 2.  If it hasn't */
	/*  found a MRCA by then, then the MRCA must be at NumEvents-1 and it automatically gets  */
	/*  there.  This helps out, because ClimbTree enters a death loop if gets */
	/*  called from the root node sometime. */
	for(i=0;i<A->NumEvents-1;i++) {
		for(j=0;j<A->Pars->NumLeaves;j++)  {
			ClimbTree( &(NodeArray[j]), A->Events[i]->time, x, OneOrZero );
		}
		if(A->Events[i]->type == Coalesce  &&
						A->Events[i]->NumThrough == A->Pars->NumLeaves)  
			break;
	}
		
	/*  deallocate NodeArray */
	free(NodeArray);
	
	
	/*  now, i is the index of the event at which the MRCA occurs  */
	return(i);
}


/*  Given a node, climb the tree, adding to NumThrough on branches and nodes */
/*  until you arrive at the first node that occurs at a time that is */
/*  strictly greater than TopTime.  Change the input node */
/*  so that it now points to the new node you are at */
void ClimbTree(node_struct **N, double TopTime, double Pos, int OneOrZero)
{
	int stop = 0;
	enum BranchNums LeftRightMiddle;
	
	
	while( (*N)->time <= TopTime  && (*N) != NULL  && !stop )  {  /*  only do anything if you are currently  */
								   /*  less than or equal to TopTime.  And stop the loop using the "stop" variable */
								   /*  in those cases when it would otherwise get you into an endless loop. */
								   
		if( (*N)->type == Coalesce  || (*N)->type == Leaf ) {  /*  node is coalescence we go up the middle branch */
			LeftRightMiddle = Middle;
		}
		else {  /*  otherwise it is a Recomb */
			/*  determine if you go up the right or the left branch */
			if(Pos <= (*N)->x)
				LeftRightMiddle = Left;
			else
				LeftRightMiddle = Right;
		}
		
		if((*N)->Br[LeftRightMiddle]->top != NULL) {  /*  only do this if there is really another node up there */
			/*  increment the NumThroughs on the middle branch above it  */
			(*N)->Br[LeftRightMiddle]->NumThrough[OneOrZero]++;
			(*N)->Br[LeftRightMiddle]->NumThrough[2]++;
			
			/*  and then proceed to the next node and increment its NumThrough too. */
			(*N) = (*N)->Br[LeftRightMiddle]->top;
			(*N)->NumThrough++;
		}
		else {
			stop = 1;
		}
	}		
}




#define SMIDGE .0000000001

int CompDoubles (const void *a, const void *b)
{
	const double *da = (const double *) a;
	const double *db = (const double *) b;

	return (*da > *db) - (*da < *db);
}



/*  Goes through and determines which of the break points affect the genealogy of the segment and */
/*  which do not.  Then it allocates space to an array of the VisiBreaks and voila! */ 
/*  This function also finds the MRCA of the Left Endpoint and the time of that  */
void FindVisiBreaks(AR_graph_struct *A)
{
	int i,j;
	int *Visi;
	int MRCA,tempMRCA;
	int *TempMRCAArray;
	
	/*  memory allocation */
	Visi = (int *)calloc(A->NumAllBreaks , sizeof(int));
	TempMRCAArray = (int *)calloc(A->NumAllBreaks , sizeof(int));
	
	for(i=0;i<A->NumAllBreaks;i++)
		Visi[i] = 0;  /*  initialize to 0, meaning No! */
	
	/*  first find the MRCA and genealogy for the left-most point.  Currently that is 0.0 */
	/*  this info will be stored in the NumThroughs[1] for the branches */
	SetNumThroughsToZero(A,1);
	SetNumThroughsToZero(A,2);  /*  also set the cumulative ones to zeroes */
	
	/*  then do the genealogy for the left endpoint, and store it in NumThroughs[1] */
	tempMRCA = FindMRCAbyPaths(A,A->Pars->Lo,1);
	A->LeftMRCA = tempMRCA;
	A->LeftTimeToMRCA = A->Events[A->LeftMRCA]->time;
	
	A->NumVisiBreaks = 0;
	for(i=0;i<A->NumAllBreaks;i++)  {
		SetNumThroughsToZero(A,i%2);
		

		MRCA = FindMRCAbyPaths(A,A->AllBreaks[i]+SMIDGE,i%2);
		TempMRCAArray[i] = MRCA;

		Visi[i] = GenealogyHasChanged(A,MRCA,tempMRCA);
		if(Visi[i]==1) 
			A->NumVisiBreaks++;
		
		tempMRCA = MRCA;
		
/* 		printf("\n\nBreakpoint %d at position %.5f\n\n",i,A->AllBreaks[i]); */
	}
	
	/* Add one more to the NumVisiBreaks to account for the left endpoint which we will
	 call a visi break even though it really isn't  */
	
	
	/*  now we go and fill up the VisiBreaks and MRCAofJunctions Arrays */
	j=0;
	A->VisiBreaks = (double *)calloc(A->NumVisiBreaks, sizeof(double));
	A->MRCAofVisiBreaks = (int *)calloc(A->NumVisiBreaks, sizeof(int)); 
	for(i=0;i<A->NumAllBreaks;i++)  {
		if(Visi[i]==1)  {
			A->VisiBreaks[j] = A->AllBreaks[i];
			A->MRCAofVisiBreaks[j] = TempMRCAArray[i];
			j++;
		}	
	}
	
	/*  free the memory allocated: */
	free(Visi);	
	free(TempMRCAArray);
}
#undef SMIDGE












/******************************************************************************************/
/*                                                                                        */
/*          FUNCTIONS TO PREPARE FOR PLOTTING THE GENEALOGY GRAPHICALLY                   */
/*                                                                                        */  
/******************************************************************************************/

/*  given that the leaves have been placed in positions, go back through the events */
/*  pointers and map the positions of the events and their caps and floors (the horizontal parts) */
void MapNodesGivenLeaves(AR_graph_struct *A) 
{
	int i;
	node_struct *Node;  /*  just temporary to clean up notation a bit */
	double rshift = .25/(double)(A->Pars->NumLeaves);
	double T = A->Root->time;
	
	CYC_EVENTS(i,A)
		Node = A->Events[i];
		if(Node->type == Coalesce) {
			/*  set the x-coordinates for the input and output branches */
			Node->P[Middle][0] = (Node->Br[Left]->bottom->P[Node->Br[Left]->BS][0] + 
								  Node->Br[Right]->bottom->P[Node->Br[Right]->BS][0]) / 2.0;
			Node->P[Left][0] = Node->Br[Left]->bottom->P[Node->Br[Left]->BS][0];
			Node->P[Right][0] = Node->Br[Right]->bottom->P[Node->Br[Right]->BS][0];
			
			/*  set the y-coordinate for the node */
			Node->Pos[1] = Node->time/T;
		}
		else if(Node->type == Recomb) {
			
			Node->P[Middle][0] = Node->Br[Middle]->bottom->P[Node->Br[Middle]->BS][0];
			Node->Pos[1] = Node->time/T;
			
			/*  then we shift the branches coming out of the right and the left by the amount rshift */
			Node->P[Left][0] = Node->P[Middle][0] - rshift;
			Node->P[Right][0] = Node->P[Middle][0] + rshift;
		}
	END1CYCLE
}



/*  Post Order Map with Position.  This is like Post Order Map, but it only tours */
/*  down branches that are active given the position Pos on the chromosome.  AND, */
/*  It maps the P's rather than the Pos's so it's a bit better for showing the recombs */
void PostOrderMapWPos(node_struct *Node, int *LV, double T, int LeafNum, int *NV, double Pos)
{
	int RightAct,LeftAct;  /*  temp variables to record which branches are active coming out of a coalescence */
	double rshift = .25/(double)(LeafNum-1);
	
	
	if(Node->type == Coalesce) {
		RightAct = 0;
		LeftAct = 0;
		Node->VisitOrder = NV[0]++;
		/*  only tour down branches that are ACTIVE, relative to the Position */
		if(Node->Br[Left] != NULL  && Node->Br[Left]->lo <= Pos && Node->Br[Left]->hi >= Pos)   {
			PostOrderMapWPos(Node->Br[Left]->bottom,LV,T,LeafNum, NV,Pos);
			LeftAct = 1;
		}
		if(Node->Br[Right] != NULL && Node->Br[Right]->lo <= Pos && Node->Br[Right]->hi >= Pos) {
			PostOrderMapWPos(Node->Br[Right]->bottom,LV,T,LeafNum, NV,Pos);
			RightAct=1;
		}
			
		/*  finally, when you've toured the left and right branches, you set the */
		/*  x and y positions of your coalescent node.  Put it right above the lower node if only */
		/*  one branch is active, and put it halfway between if both branches are active. */
		if(RightAct == 1 && LeftAct == 1) {
			Node->P[Middle][0] = (Node->Br[Left]->bottom->P[Node->Br[Left]->BS][0] + 
								  Node->Br[Right]->bottom->P[Node->Br[Right]->BS][0]) / 2.0;
			Node->P[Left][0] = Node->Br[Left]->bottom->P[Node->Br[Left]->BS][0];
			Node->P[Right][0] = Node->Br[Right]->bottom->P[Node->Br[Right]->BS][0];
		}
		else if(RightAct == 1)  {
			Node->P[Middle][0] = Node->Br[Right]->bottom->P[Node->Br[Right]->BS][0];
			Node->P[Right][0] = Node->Br[Right]->bottom->P[Node->Br[Right]->BS][0];
			Node->P[Left][0] = Node->Br[Right]->bottom->P[Node->Br[Right]->BS][0];
		}
		else if(LeftAct == 1) {
			Node->P[Middle][0] = Node->Br[Left]->bottom->P[Node->Br[Left]->BS][0];
			Node->P[Right][0] = Node->Br[Left]->bottom->P[Node->Br[Left]->BS][0];
			Node->P[Left][0] = Node->Br[Left]->bottom->P[Node->Br[Left]->BS][0];
		}
		Node->Pos[1] = Node->time/T;
/* 		printf("\nReached Coalescent Node Number %d, at Xpos = %f, Ypos = %f",Node->ID, */
/* 				Node->Pos[0], Node->Pos[1]); */
	} 
	
	/*  otherwise, if it is a recombination... */
	else if(Node->type == Recomb) {  

		/*  ...and if this is the first time you've reached it... */
		if(Node->BeenVisited==0)  {
			
			Node->VisitOrder = NV[0]++;
			/*  let us know we've been to it  (IN FACT, we should only visit each of these once anyway if we are only following */
												/*  active branches ) */
			Node->BeenVisited++;
		
			if(Node->Br[Middle] != NULL)  {  /*  then tour down the middle  / */
				PostOrderMapWPos(Node->Br[Middle]->bottom,LV,T,LeafNum, NV,Pos);
				
				/*  after we've returned from our tour down the middle branch, we must set the */
				/*  middle x position to be directly above the next node down, and the y-position to be */
				/*  where it ought to be. and the left and right positions to be r-shifted. */
				Node->P[Middle][0] = Node->Br[Middle]->bottom->P[Node->Br[Middle]->BS][0];
				Node->Pos[1] = Node->time/T;
				
				/*  then we shift the branches coming out of the right and the left by the amount rshift */
				Node->P[Left][0] = Node->P[Middle][0] - rshift;
				Node->P[Right][0] = Node->P[Middle][0] + rshift;
				
/* 				printf("\nReached Recomb Node Number %d, at Xpos = %f, Ypos = %f",Node->ID, */
/* 				Node->Pos[0], Node->Pos[1]); */
				
			}
			else  /*  print an error if the Middle was NULL on the Recomb node */
				printf("\n\nBummer Dude! Middle Branch is NULL on a Recomb!\n\n");
		}
		
		/*  if we've already visited the node, we don't do anything, just return control */
		/*  back to the calling function. */
		
	}
	
	
	else if(Node->type==Leaf)  {  /*  if we have reached a leaf, then map its position in 2-space */
		
		Node->VisitOrder = NV[0]++;
		
		/*  then set its x position */
		Node->P[Middle][0] = (double)(*LV) * 1.0/(double)(LeafNum-1);  /*  x position */
		/*  set the left and the right to the same on these too */
		Node->P[Left][0] = Node->P[Middle][0];
		Node->P[Right][0] = Node->P[Middle][0];
		
		
		Node->Pos[1] = 0.0;							   /*  y position */
/* 		printf("\nReached Node Number %d, Called Leaf Number %d at Xpos = %f",Node->ID,*LV,Node->Pos[0]); */
		
		(*LV) += 1;
	}
	
}





/*  To map out the nodes relevant only to a binary tree given a particular position on the chromosome */
void MapNodesWPos(AR_graph_struct *A, double Pos)
{
	int leaves_visited = 0;
	int nodes_visited = 0;
	int i;
	int Start;
	
	/*  first, set the BeenVisited variable at all nodes to 0 */
	CYC_EVENTS(i,A)
		A->Events[i]->BeenVisited=0;
	END1CYCLE
	
	/*  then go down the Events array until hitting the first coalescent node that has two active branches coming out of it */
	for(Start=A->NumEvents-1;Start>0;Start--) {
		if(A->Events[Start]->type == Coalesce  && 
					(A->Events[Start]->Br[Left]->lo <= Pos && A->Events[Start]->Br[Left]->hi >= Pos)  &&
					(A->Events[Start]->Br[Right]->lo <= Pos && A->Events[Start]->Br[Right]->hi >= Pos) )
			break;
	}
	
	PostOrderMapWPos(A->Events[Start], &leaves_visited, A->Root->time, A->Pars->NumLeaves,
			&nodes_visited, Pos);

}

